@@ -36,15 +36,18 @@
 #include <ofono/modem.h>
 #include <ofono/gprs.h>
 
+#include <drivers/atmodem/atutil.h>
+
 #include "gatchat.h"
 #include "gatresult.h"
 
-#include "atmodem.h"
+#include "common.h"
 #include "vendor.h"
 
 #define MAX_CONTEXTS 255
 
 static const char *cgreg_prefix[] = { "+CGREG:", NULL };
+static const char *cgerep_prefix[] = { "+CGEREP:", NULL };
 static const char *cgdcont_prefix[] = { "+CGDCONT:", NULL };
 static const char *cgact_prefix[] = { "+CGACT:", NULL };
 static const char *none_prefix[] = { NULL };
@@ -542,6 +545,54 @@ static void telit_mode_notify(GAtResult *result, gpointer user_data)
 	ofono_gprs_bearer_notify(gprs, bearer);
 }
 
+static void simcom_mode_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_gprs *gprs = user_data;
+	GAtResultIter iter;
+	gint stat, bearer;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+CNSMOD:"))
+		return;
+
+	if (!g_at_result_iter_next_number(&iter,&stat))
+		return;
+
+	switch (stat) {
+	case 0:
+		bearer = PACKET_BEARER_NONE;
+		break;
+	case 1:
+	case 2:
+		bearer = PACKET_BEARER_GPRS;
+		break;
+	case 3:
+		bearer = PACKET_BEARER_EGPRS;
+		break;
+	case 4:
+		bearer = PACKET_BEARER_UMTS;
+		break;
+	case 5:
+		bearer = PACKET_BEARER_HSDPA;
+		break;
+	case 6:
+		bearer = PACKET_BEARER_HSUPA;
+		break;
+	case 7:
+		bearer = PACKET_BEARER_HSUPA_HSDPA;
+		break;
+	case 8:
+		bearer = PACKET_BEARER_EPS;
+		break;
+	default:
+		bearer = PACKET_BEARER_NONE;
+		break;
+	}
+
+	ofono_gprs_bearer_notify(gprs, bearer);
+}
+
 static void ublox_ureg_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_gprs *gprs = user_data;
@@ -624,6 +675,15 @@ static void gprs_initialized(gboolean ok, GAtResult *result, gpointer user_data)
 		g_at_chat_send(gd->chat, "AT#PSNT=1", none_prefix,
 						NULL, NULL, NULL);
 		break;
+	case OFONO_VENDOR_SIMCOM:
+		g_at_chat_register(gd->chat, "+CNSMOD:", simcom_mode_notify,
+						FALSE, gprs, NULL);
+		g_at_chat_send(gd->chat, "AT+CNSMOD=1", none_prefix,
+						NULL, NULL, NULL);
+		break;
+	case OFONO_VENDOR_QUECTEL_EC2X:
+	case OFONO_VENDOR_QUECTEL_SERIAL:
+		break;
 	default:
 		g_at_chat_register(gd->chat, "+CPSB:", cpsb_notify,
 						FALSE, gprs, NULL);
@@ -645,6 +705,65 @@ static void gprs_initialized(gboolean ok, GAtResult *result, gpointer user_data)
 	ofono_gprs_register(gprs);
 }
 
+static void at_cgerep_test_cb(gboolean ok, GAtResult *result,
+				gpointer user_data)
+{
+	struct ofono_gprs *gprs = user_data;
+	struct gprs_data *gd = ofono_gprs_get_data(gprs);
+	GAtResultIter iter;
+	int min, max, arg1 = 0, arg2 = 0;
+	gboolean two_arguments = true;
+	char buf[20];
+
+	if (!ok) {
+		ofono_error("Error querying AT+CGEREP=? Failing...");
+		ofono_gprs_remove(gprs);
+		return;
+	}
+
+	g_at_result_iter_init(&iter, result);
+
+	g_at_result_iter_next(&iter, "+CGEREP:");
+
+	if (!g_at_result_iter_open_list(&iter)) {
+		ofono_error("Malformed reply from AT+CGEREP=? Failing...");
+		ofono_gprs_remove(gprs);
+		return;
+	}
+
+	while (g_at_result_iter_next_range(&iter, &min, &max)) {
+		if ((min <= 1) && (max >= 1))
+			arg1 = 1;
+
+		if ((min <= 2) && (max >= 2))
+			arg1 = 2;
+	}
+
+	if (!g_at_result_iter_close_list(&iter))
+		goto out;
+
+	if (!g_at_result_iter_open_list(&iter)) {
+		two_arguments = false;
+		goto out;
+	}
+
+	while (g_at_result_iter_next_range(&iter, &min, &max)) {
+		if ((min <= 1) && (max >= 1))
+			arg2 = 1;
+	}
+
+	g_at_result_iter_close_list(&iter);
+
+out:
+	if (two_arguments)
+		sprintf(buf, "AT+CGEREP=%u,%u", arg1, arg2);
+	else
+		sprintf(buf, "AT+CGEREP=%u", arg1);
+
+	g_at_chat_send(gd->chat, buf, none_prefix, gprs_initialized, gprs,
+		NULL);
+}
+
 static void at_cgreg_test_cb(gboolean ok, GAtResult *result,
 				gpointer user_data)
 {
@@ -685,7 +804,10 @@ retry:
 		goto error;
 
 	g_at_chat_send(gd->chat, cmd, none_prefix, NULL, NULL, NULL);
-	g_at_chat_send(gd->chat, "AT+CGAUTO=0", none_prefix, NULL, NULL, NULL);
+
+	if (gd->vendor != OFONO_VENDOR_SIMCOM)
+		g_at_chat_send(gd->chat, "AT+CGAUTO=0", none_prefix,
+					NULL, NULL, NULL);
 
 	switch (gd->vendor) {
 	case OFONO_VENDOR_MBM:
@@ -699,8 +821,8 @@ retry:
 			gprs_initialized, gprs, NULL);
 		break;
 	default:
-		g_at_chat_send(gd->chat, "AT+CGEREP=2,1", none_prefix,
-			gprs_initialized, gprs, NULL);
+		g_at_chat_send(gd->chat, "AT+CGEREP=?", cgerep_prefix,
+			at_cgerep_test_cb, gprs, NULL);
 		break;
 	}
 
@@ -800,7 +922,6 @@ static void at_gprs_remove(struct ofono_gprs *gprs)
 }
 
 static const struct ofono_gprs_driver driver = {
-	.name			= "atmodem",
 	.probe			= at_gprs_probe,
 	.remove			= at_gprs_remove,
 	.set_attached		= at_gprs_set_attached,
@@ -808,12 +929,4 @@ static const struct ofono_gprs_driver driver = {
 	.list_active_contexts	= at_gprs_list_active_contexts,
 };
 
-void at_gprs_init(void)
-{
-	ofono_gprs_driver_register(&driver);
-}
-
-void at_gprs_exit(void)
-{
-	ofono_gprs_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(gprs, atmodem, &driver)
