@@ -34,14 +34,14 @@
 #include <ofono/modem.h>
 #include <ofono/sim.h>
 
+#include <drivers/atmodem/atutil.h>
+
 #include "gatchat.h"
 #include "gatresult.h"
 #include "simutil.h"
 #include "vendor.h"
 #include "util.h"
 
-#include "atmodem.h"
-
 #define EF_STATUS_INVALIDATED 0
 #define EF_STATUS_VALID 1
 
@@ -1640,7 +1640,17 @@ static void at_discover_apps(struct ofono_sim *sim,
 				void *data)
 {
 	struct sim_data *sd = ofono_sim_get_data(sim);
-	struct cb_data *cbd = cb_data_new(cb, data);
+	struct cb_data *cbd;
+
+	/*
+	 * QUECTEL EC2X and SIMCom A7605E-H reboots when executing the AT+CUAD
+	 * command with SIM cards of some operators
+	 */
+	if (sd->vendor == OFONO_VENDOR_QUECTEL_EC2X
+			|| sd->vendor == OFONO_VENDOR_SIMCOM)
+		goto error;
+
+	cbd = cb_data_new(cb, data);
 
 	if (g_at_chat_send(sd->chat, "AT+CUAD", cuad_prefix,
 			at_discover_apps_cb, cbd, g_free) > 0)
@@ -1648,6 +1658,7 @@ static void at_discover_apps(struct ofono_sim *sim,
 
 	g_free(cbd);
 
+error:
 	CALLBACK_WITH_FAILURE(cb, NULL, 0, data);
 }
 
@@ -2082,7 +2093,6 @@ static void at_set_active_card_slot(struct ofono_sim *sim, unsigned int index,
 }
 
 static const struct ofono_sim_driver driver = {
-	.name			= "atmodem",
 	.probe			= at_sim_probe,
 	.remove			= at_sim_remove,
 	.read_file_info		= at_sim_read_info,
@@ -2110,28 +2120,4 @@ static const struct ofono_sim_driver driver = {
 	.set_active_card_slot	= at_set_active_card_slot
 };
 
-static const struct ofono_sim_driver driver_noef = {
-	.name			= "atmodem-noef",
-	.probe			= at_sim_probe,
-	.remove			= at_sim_remove,
-	.read_imsi		= at_read_imsi,
-	.query_passwd_state	= at_pin_query,
-	.query_pin_retries	= at_pin_retries_query,
-	.send_passwd		= at_pin_send,
-	.reset_passwd		= at_pin_send_puk,
-	.lock			= at_pin_enable,
-	.change_passwd		= at_change_passwd,
-	.query_facility_lock	= at_query_clck,
-};
-
-void at_sim_init(void)
-{
-	ofono_sim_driver_register(&driver);
-	ofono_sim_driver_register(&driver_noef);
-}
-
-void at_sim_exit(void)
-{
-	ofono_sim_driver_unregister(&driver);
-	ofono_sim_driver_unregister(&driver_noef);
-}
+OFONO_ATOM_DRIVER_BUILTIN(sim, atmodem, &driver)
