@@ -280,19 +280,10 @@ static void mbim_gprs_remove(struct ofono_gprs *gprs)
 }
 
 static const struct ofono_gprs_driver driver = {
-	.name			= "mbim",
 	.probe			= mbim_gprs_probe,
 	.remove			= mbim_gprs_remove,
 	.set_attached		= mbim_gprs_set_attached,
 	.attached_status	= mbim_gprs_registration_status,
 };
 
-void mbim_gprs_init(void)
-{
-	ofono_gprs_driver_register(&driver);
-}
-
-void mbim_gprs_exit(void)
-{
-	ofono_gprs_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(gprs, mbim, &driver)
