@@ -34,10 +34,11 @@
 #include <ofono/modem.h>
 #include <ofono/gprs-context.h>
 
+#include <drivers/atmodem/atutil.h>
+
 #include "gatchat.h"
 #include "gatresult.h"
 
-#include "mbmmodem.h"
 #include "src/missing.h"
 
 #define MBM_E2NAP_DISCONNECTED 0
@@ -135,12 +136,12 @@ out:
 	modem = ofono_gprs_context_get_modem(gc);
 	interface = ofono_modem_get_string(modem, "NetworkInterface");
 
-	ofono_info("IP: %s  Gateway: %s", ip, gateway);
-	ofono_info("DNS: %s, %s", dns[0], dns[1]);
-
 	ofono_gprs_context_set_interface(gc, interface);
 
 	if (success) {
+		ofono_info("IP: %s  Gateway: %s", ip, gateway);
+		ofono_info("DNS: %s, %s", dns[0], dns[1]);
+
 		ofono_gprs_context_set_ipv4_address(gc, ip, TRUE);
 		ofono_gprs_context_set_ipv4_netmask(gc, STATIC_IP_NETMASK);
 		ofono_gprs_context_set_ipv4_dns_servers(gc, dns);
@@ -507,19 +508,10 @@ static void mbm_gprs_context_remove(struct ofono_gprs_context *gc)
 }
 
 static const struct ofono_gprs_context_driver driver = {
-	.name			= "mbmmodem",
 	.probe			= mbm_gprs_context_probe,
 	.remove			= mbm_gprs_context_remove,
 	.activate_primary	= mbm_gprs_activate_primary,
 	.deactivate_primary	= mbm_gprs_deactivate_primary,
 };
 
-void mbm_gprs_context_init(void)
-{
-	ofono_gprs_context_driver_register(&driver);
-}
-
-void mbm_gprs_context_exit(void)
-{
-	ofono_gprs_context_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(gprs_context, mbmmodem, &driver)
