@@ -32,8 +32,7 @@
 #include "qmi.h"
 #include "dms.h"
 #include "uim.h"
-
-#include "qmimodem.h"
+#include "util.h"
 #include "simutil.h"
 
 #define EF_STATUS_INVALIDATED 0
@@ -64,6 +63,7 @@ struct sim_data {
 	uint8_t app_type;
 	uint32_t retry_count;
 	guint poll_source;
+	uint16_t card_status_indication_id;
 };
 
 static void qmi_query_passwd_state(struct ofono_sim *sim,
@@ -183,8 +183,6 @@ static void qmi_read_attributes(struct ofono_sim *sim, int fileid,
 		goto error;
 
 	param = qmi_param_new();
-	if (!param)
-		goto error;
 
 	qmi_param_append(param, 0x01, sizeof(aid_data), aid_data);
 	qmi_param_append(param, 0x02, fileid_len, fileid_data);
@@ -198,7 +196,6 @@ static void qmi_read_attributes(struct ofono_sim *sim, int fileid,
 error:
 	CALLBACK_WITH_FAILURE(cb, -1, -1, -1, NULL,
 					EF_STATUS_INVALIDATED, cbd->data);
-
 	g_free(cbd);
 }
 
@@ -252,8 +249,6 @@ static void qmi_read_transparent(struct ofono_sim *sim,
 	read_data[3] = (length & 0xff00) >> 8;
 
 	param = qmi_param_new();
-	if (!param)
-		goto error;
 
 	qmi_param_append(param, 0x01, sizeof(aid_data), aid_data);
 	qmi_param_append(param, 0x02, fileid_len, fileid_data);
@@ -267,7 +262,6 @@ static void qmi_read_transparent(struct ofono_sim *sim,
 
 error:
 	CALLBACK_WITH_FAILURE(cb, NULL, 0, user_data);
-
 	g_free(cbd);
 }
 
@@ -298,8 +292,6 @@ static void qmi_read_record(struct ofono_sim *sim,
 	read_data[3] = (length & 0xff00) >> 8;
 
 	param = qmi_param_new();
-	if (!param)
-		goto error;
 
 	qmi_param_append(param, 0x01, sizeof(aid_data), aid_data);
 	qmi_param_append(param, 0x02, fileid_len, fileid_data);
@@ -313,7 +305,6 @@ static void qmi_read_record(struct ofono_sim *sim,
 
 error:
 	CALLBACK_WITH_FAILURE(cb, NULL, 0, user_data);
-
 	g_free(cbd);
 }
 
@@ -383,8 +374,6 @@ static void write_generic(struct ofono_sim *sim,
 	memcpy(&write_data[4], value, length);
 
 	param = qmi_param_new();
-	if (!param)
-		goto error;
 
 	qmi_param_append(param, 0x01, sizeof(aid_data), aid_data);
 	qmi_param_append(param, 0x02, fileid_len, fileid_data);
@@ -398,7 +387,6 @@ static void write_generic(struct ofono_sim *sim,
 
 error:
 	CALLBACK_WITH_FAILURE(cb, user_data);
-
 	g_free(cbd);
 }
 
@@ -745,8 +733,6 @@ static void qmi_pin_send(struct ofono_sim *sim, const char *passwd,
 		goto error;
 
 	param = qmi_param_new();
-	if (!param)
-		goto error;
 
 	/* param info */
 	info_data = alloca(2 + passwd_len);
@@ -769,7 +755,6 @@ static void qmi_pin_send(struct ofono_sim *sim, const char *passwd,
 
 error:
 	CALLBACK_WITH_FAILURE(cb, cbd->data);
-
 	g_free(cbd);
 }
 
@@ -844,9 +829,13 @@ static void event_registration_cb(struct qmi_result *result, void *user_data)
 		goto error;
 
 	DBG("event mask 0x%04x", data->event_mask);
-	if (data->event_mask & 0x0001)
-		qmi_service_register(data->uim, QMI_UIM_GET_CARD_STATUS_EVENT,
+
+	if (data->event_mask & 0x0001) {
+		data->card_status_indication_id =
+			qmi_service_register(data->uim,
+						QMI_UIM_GET_CARD_STATUS_EVENT,
 						card_status_notify, sim, NULL);
+	}
 
 	if (qmi_service_send(data->uim, QMI_UIM_GET_CARD_STATUS, NULL,
 					get_card_status_cb, sim, NULL) > 0)
@@ -856,7 +845,6 @@ error:
 	ofono_sim_remove(sim);
 }
 
-
 static void create_uim_cb(struct qmi_service *service, void *user_data)
 {
 	struct ofono_sim *sim = user_data;
@@ -874,8 +862,6 @@ static void create_uim_cb(struct qmi_service *service, void *user_data)
 	data->uim = qmi_service_ref(service);
 
 	param = qmi_param_new_uint32(QMI_UIM_PARAM_EVENT_MASK, mask);
-	if (!param)
-		goto error;
 
 	if (qmi_service_send(data->uim, QMI_UIM_EVENT_REGISTRATION, param,
 					event_registration_cb, sim, NULL) > 0)
@@ -936,20 +922,25 @@ static void qmi_sim_remove(struct ofono_sim *sim)
 		g_source_remove(data->poll_source);
 
 	if (data->uim) {
-		qmi_service_unregister_all(data->uim);
+		if (data->card_status_indication_id) {
+			qmi_service_unregister(data->uim,
+					data->card_status_indication_id);
+			data->card_status_indication_id = 0;
+		}
+
 		qmi_service_unref(data->uim);
 		data->uim = NULL;
 	}
+
 	if (data->dms) {
-		qmi_service_unregister_all(data->dms);
 		qmi_service_unref(data->dms);
+		data->dms = NULL;
 	}
 
 	g_free(data);
 }
 
 static const struct ofono_sim_driver driver = {
-	.name			= "qmimodem",
 	.probe			= qmi_sim_probe,
 	.remove			= qmi_sim_remove,
 	.read_file_info		= qmi_read_attributes,
@@ -965,12 +956,4 @@ static const struct ofono_sim_driver driver = {
 	.send_passwd		= qmi_pin_send,
 };
 
-void qmi_sim_init(void)
-{
-	ofono_sim_driver_register(&driver);
-}
-
-void qmi_sim_exit(void)
-{
-	ofono_sim_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(sim, qmimodem, &driver)
