@@ -36,7 +36,6 @@ typedef void (*ofono_call_barring_query_cb_t)(const struct ofono_error *error,
 					int status, void *data);
 
 struct ofono_call_barring_driver {
-	const char *name;
 	int (*probe)(struct ofono_call_barring *cb, unsigned int vendor,
 			void *data);
 	void (*remove)(struct ofono_call_barring *cb);
@@ -50,11 +49,6 @@ struct ofono_call_barring_driver {
 			ofono_call_barring_set_cb_t cb, void *data);
 };
 
-int ofono_call_barring_driver_register(
-				const struct ofono_call_barring_driver *d);
-void ofono_call_barring_driver_unregister(
-				const struct ofono_call_barring_driver *d);
-
 struct ofono_call_barring *ofono_call_barring_create(struct ofono_modem *modem,
 							unsigned int vendor,
 							const char *driver,
