@@ -27,6 +27,8 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <stdio.h>
+#include <net/if.h>
 
 #define OFONO_API_SUBJECT_TO_CHANGE
 #include <ofono/plugin.h>
@@ -48,6 +50,8 @@
 #include <ofono/log.h>
 #include <ofono/message-waiting.h>
 
+#include <ell/ell.h>
+
 #include <drivers/qmimodem/qmi.h>
 #include <drivers/qmimodem/dms.h>
 #include <drivers/qmimodem/wda.h>
@@ -68,9 +72,16 @@
 struct gobi_data {
 	struct qmi_device *device;
 	struct qmi_service *dms;
+	struct qmi_service *wda;
 	unsigned long features;
 	unsigned int discover_attempts;
 	uint8_t oper_mode;
+	bool using_mux;
+	bool using_qmi_wwan_q;
+	int main_net_ifindex;
+	char main_net_name[IFNAMSIZ];
+	uint32_t max_aggregation_size;
+	uint32_t set_powered_id;
 };
 
 static void gobi_debug(const char *str, void *user_data)
@@ -83,6 +94,7 @@ static void gobi_debug(const char *str, void *user_data)
 static int gobi_probe(struct ofono_modem *modem)
 {
 	struct gobi_data *data;
+	const char *kernel_driver;
 
 	DBG("%p", modem);
 
@@ -90,11 +102,33 @@ static int gobi_probe(struct ofono_modem *modem)
 	if (!data)
 		return -ENOMEM;
 
+	kernel_driver = ofono_modem_get_string(modem, "KernelDriver");
+	DBG("kernel_driver: %s", kernel_driver);
+
+	if (!strcmp(kernel_driver, "qmi_wwan_q"))
+		data->using_qmi_wwan_q = true;
+
+	data->main_net_ifindex =
+		ofono_modem_get_integer(modem, "NetworkInterfaceIndex");
+	l_strlcpy(data->main_net_name,
+			ofono_modem_get_string(modem, "NetworkInterface"),
+			sizeof(data->main_net_name));
+	DBG("net: %s (%d)", data->main_net_name, data->main_net_ifindex);
+
 	ofono_modem_set_data(modem, data);
 
 	return 0;
 }
 
+static void cleanup_services(struct gobi_data *data)
+{
+	qmi_service_unref(data->dms);
+	data->dms = NULL;
+
+	qmi_service_unref(data->wda);
+	data->wda = NULL;
+}
+
 static void gobi_remove(struct ofono_modem *modem)
 {
 	struct gobi_data *data = ofono_modem_get_data(modem);
@@ -103,7 +137,12 @@ static void gobi_remove(struct ofono_modem *modem)
 
 	ofono_modem_set_data(modem, NULL);
 
-	qmi_service_unref(data->dms);
+	if (data->set_powered_id) {
+		l_netlink_cancel(l_rtnl_get(), data->set_powered_id);
+		data->set_powered_id = 0;
+	}
+
+	cleanup_services(data);
 
 	qmi_device_unref(data->device);
 
@@ -131,8 +170,7 @@ static void shutdown_device(struct ofono_modem *modem)
 
 	DBG("%p", modem);
 
-	qmi_service_unref(data->dms);
-	data->dms = NULL;
+	cleanup_services(data);
 
 	qmi_device_shutdown(data->device, shutdown_cb, modem, NULL);
 }
@@ -234,6 +272,75 @@ error:
 	shutdown_device(modem);
 }
 
+static void get_data_format_cb(struct qmi_result *result, void *user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct gobi_data *data = ofono_modem_get_data(modem);
+	uint32_t llproto;
+	enum qmi_device_expected_data_format expected_llproto;
+
+	DBG("");
+
+	if (qmi_result_set_error(result, NULL))
+		goto done;
+
+	if (!qmi_result_get_uint32(result, QMI_WDA_LL_PROTOCOL, &llproto))
+		goto done;
+
+	expected_llproto = qmi_device_get_expected_data_format(data->device);
+
+	if ((llproto == QMI_WDA_DATA_LINK_PROTOCOL_802_3) &&
+			(expected_llproto ==
+				QMI_DEVICE_EXPECTED_DATA_FORMAT_RAW_IP)) {
+		if (!qmi_device_set_expected_data_format(data->device,
+					QMI_DEVICE_EXPECTED_DATA_FORMAT_802_3))
+			DBG("Fail to set expected data to 802.3");
+		else
+			DBG("expected data set to 802.3");
+	} else if ((llproto == QMI_WDA_DATA_LINK_PROTOCOL_RAW_IP) &&
+			(expected_llproto ==
+				QMI_DEVICE_EXPECTED_DATA_FORMAT_802_3)) {
+		if (!qmi_device_set_expected_data_format(data->device,
+					QMI_DEVICE_EXPECTED_DATA_FORMAT_RAW_IP))
+			DBG("Fail to set expected data to raw-ip");
+		else
+			DBG("expected data set to raw-ip");
+	}
+
+done:
+	if (qmi_service_send(data->dms, QMI_DMS_GET_CAPS, NULL,
+						get_caps_cb, modem, NULL) > 0)
+		return;
+
+	shutdown_device(modem);
+}
+
+static void create_wda_cb(struct qmi_service *service, void *user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct gobi_data *data = ofono_modem_get_data(modem);
+
+	DBG("");
+
+	if (!service) {
+		DBG("Failed to request WDA service, continue initialization");
+		goto error;
+	}
+
+	data->wda = qmi_service_ref(service);
+
+	if (qmi_service_send(data->wda, QMI_WDA_GET_DATA_FORMAT, NULL,
+				get_data_format_cb, modem, NULL) > 0)
+		return;
+
+error:
+	if (qmi_service_send(data->dms, QMI_DMS_GET_CAPS, NULL,
+						get_caps_cb, modem, NULL) > 0)
+		return;
+
+	shutdown_device(modem);
+}
+
 static void create_dms_cb(struct qmi_service *service, void *user_data)
 {
 	struct ofono_modem *modem = user_data;
@@ -246,8 +353,8 @@ static void create_dms_cb(struct qmi_service *service, void *user_data)
 
 	data->dms = qmi_service_ref(service);
 
-	if (qmi_service_send(data->dms, QMI_DMS_GET_CAPS, NULL,
-					get_caps_cb, modem, NULL) > 0)
+	if (qmi_service_create(data->device, QMI_SERVICE_WDA,
+					create_wda_cb, modem, NULL))
 		return;
 
 error:
@@ -400,35 +507,112 @@ static void set_online_cb(struct qmi_result *result, void *user_data)
 		CALLBACK_WITH_SUCCESS(cb, cbd->data);
 }
 
+static void powered_up_cb(int error, uint16_t type,
+				const void *msg, uint32_t len,
+				void *user_data)
+{
+	struct cb_data *cbd = user_data;
+	struct gobi_data *data = cbd->user;
+	struct qmi_param *param;
+	ofono_modem_online_cb_t cb = cbd->cb;
+
+	DBG("error: %d", error);
+
+	data->set_powered_id = 0;
+
+	if (error)
+		goto error;
+
+	param = qmi_param_new_uint8(QMI_DMS_PARAM_OPER_MODE,
+						QMI_DMS_OPER_MODE_ONLINE);
+	if (!param)
+		goto error;
+
+	if (data->using_qmi_wwan_q)
+		l_sysctl_set_u32(1, "/sys/class/net/%s/link_state",
+					data->main_net_name);
+
+	cb_data_ref(cbd);
+
+	if (qmi_service_send(data->dms, QMI_DMS_SET_OPER_MODE, param,
+				set_online_cb, cbd, cb_data_unref) > 0)
+		return;
+
+	qmi_param_free(param);
+	cb_data_unref(cbd);
+error:
+	CALLBACK_WITH_FAILURE(cb, cbd->data);
+}
+
+static void powered_down_cb(int error, uint16_t type,
+				const void *msg, uint32_t len,
+				void *user_data)
+{
+	struct cb_data *cbd = user_data;
+	struct gobi_data *data = cbd->user;
+	struct qmi_param *param;
+	ofono_modem_online_cb_t cb = cbd->cb;
+
+	DBG("error: %d", error);
+
+	data->set_powered_id = 0;
+
+	if (error)
+		goto error;
+
+	param = qmi_param_new_uint8(QMI_DMS_PARAM_OPER_MODE,
+					QMI_DMS_OPER_MODE_LOW_POWER);
+	if (!param)
+		goto error;
+
+	if (data->using_qmi_wwan_q)
+		l_sysctl_set_u32(0, "/sys/class/net/%s/link_state",
+					data->main_net_name);
+
+	cb_data_ref(cbd);
+
+	if (qmi_service_send(data->dms, QMI_DMS_SET_OPER_MODE, param,
+				set_online_cb, cbd, cb_data_unref) > 0)
+		return;
+
+	qmi_param_free(param);
+	cb_data_unref(cbd);
+error:
+	CALLBACK_WITH_FAILURE(cb, cbd->data);
+}
+
 static void gobi_set_online(struct ofono_modem *modem, ofono_bool_t online,
 				ofono_modem_online_cb_t cb, void *user_data)
 {
 	struct gobi_data *data = ofono_modem_get_data(modem);
+	struct l_netlink *rtnl = l_rtnl_get();
 	struct cb_data *cbd = cb_data_new(cb, user_data);
-	struct qmi_param *param;
-	uint8_t mode;
+	l_netlink_command_func_t powered_cb;
 
-	DBG("%p %s", modem, online ? "online" : "offline");
+	DBG("%p %s using_mux: %s", modem, online ? "online" : "offline",
+		data->using_mux ? "yes" : "no");
+
+	cbd->user = data;
 
 	if (online)
-		mode = QMI_DMS_OPER_MODE_ONLINE;
+		powered_cb = powered_up_cb;
 	else
-		mode = QMI_DMS_OPER_MODE_LOW_POWER;
+		powered_cb = powered_down_cb;
 
-	param = qmi_param_new_uint8(QMI_DMS_PARAM_OPER_MODE, mode);
-	if (!param)
-		goto error;
+	if (!data->using_mux) {
+		powered_cb(0, 0, NULL, 0, cbd);
+		cb_data_unref(cbd);
+		return;
+	}
 
-	if (qmi_service_send(data->dms, QMI_DMS_SET_OPER_MODE, param,
-					set_online_cb, cbd, g_free) > 0)
+	data->set_powered_id = l_rtnl_set_powered(rtnl, data->main_net_ifindex,
+							online, powered_cb, cbd,
+							cb_data_unref);
+	if (data->set_powered_id)
 		return;
 
-	qmi_param_free(param);
-
-error:
-	CALLBACK_WITH_FAILURE(cb, cbd->data);
-
-	g_free(cbd);
+	cb_data_unref(cbd);
+	CALLBACK_WITH_FAILURE(cb, user_data);
 }
 
 static void gobi_pre_sim(struct ofono_modem *modem)
@@ -443,10 +627,10 @@ static void gobi_pre_sim(struct ofono_modem *modem)
 	if (data->features & GOBI_UIM)
 		sim_driver = "qmimodem";
 	else if (data->features & GOBI_DMS)
-		sim_driver = "qmimodem-legacy";
+		sim_driver = "qmimodem_legacy";
 
 	if (ofono_modem_get_boolean(modem, "ForceSimLegacy"))
-		sim_driver = "qmimodem-legacy";
+		sim_driver = "qmimodem_legacy";
 
 	if (sim_driver)
 		ofono_sim_create(modem, 0, sim_driver, data->device);
@@ -459,6 +643,70 @@ static void gobi_pre_sim(struct ofono_modem *modem)
 							data->device);
 }
 
+static void gobi_setup_gprs(struct ofono_modem *modem)
+{
+	struct gobi_data *data = ofono_modem_get_data(modem);
+	int n_premux = ofono_modem_get_integer(modem, "NumPremuxInterfaces");
+	struct ofono_gprs *gprs;
+	struct ofono_gprs_context *gc;
+	const char *interface;
+	char buf[256];
+	int i;
+
+	gprs = ofono_gprs_create(modem, 0, "qmimodem", data->device);
+	if (!gprs) {
+		ofono_warn("Unable to create gprs for: %s",
+					ofono_modem_get_path(modem));
+		return;
+	}
+
+	/* Simple case of 802.3 interface, no QMAP */
+	if (n_premux == 0) {
+		interface = ofono_modem_get_string(modem, "NetworkInterface");
+
+		gc = ofono_gprs_context_create(modem, 0, "qmimodem",
+							data->device);
+		if (!gc) {
+			ofono_warn("Unable to create gprs-context for: %s",
+					ofono_modem_get_path(modem));
+			return;
+		}
+
+		ofono_gprs_add_context(gprs, gc);
+		ofono_gprs_context_set_interface(gc, interface);
+
+		return;
+	}
+
+	data->using_mux = true;
+
+	data->max_aggregation_size =
+		ofono_modem_get_integer(modem, "MaxAggregationSize");
+	DBG("max_aggregation_size: %u", data->max_aggregation_size);
+
+	for (i = 0; i < n_premux; i++) {
+		int mux_id;
+
+		sprintf(buf, "PremuxInterface%dMuxId", i + 1);
+		mux_id = ofono_modem_get_integer(modem, buf);
+
+		gc = ofono_gprs_context_create(modem, mux_id, "qmimodem",
+							data->device);
+
+		if (!gc) {
+			ofono_warn("gprs-context creation failed for [%d] %s",
+					i + 1, ofono_modem_get_path(modem));
+			continue;
+		}
+
+		sprintf(buf, "PremuxInterface%d", i + 1);
+		interface = ofono_modem_get_string(modem, buf);
+
+		ofono_gprs_add_context(gprs, gc);
+		ofono_gprs_context_set_interface(gc, interface);
+	}
+}
+
 static void gobi_post_sim(struct ofono_modem *modem)
 {
 	struct gobi_data *data = ofono_modem_get_data(modem);
@@ -489,13 +737,14 @@ static void gobi_post_sim(struct ofono_modem *modem)
 		if (mw)
 			ofono_message_waiting_register(mw);
 	}
+
+	if (data->features & GOBI_WDS)
+		gobi_setup_gprs(modem);
 }
 
 static void gobi_post_online(struct ofono_modem *modem)
 {
 	struct gobi_data *data = ofono_modem_get_data(modem);
-	struct ofono_gprs *gprs;
-	struct ofono_gprs_context *gc;
 
 	DBG("%p", modem);
 
@@ -506,15 +755,6 @@ static void gobi_post_online(struct ofono_modem *modem)
 
 	if (data->features & GOBI_VOICE)
 		ofono_ussd_create(modem, 0, "qmimodem", data->device);
-
-	if (data->features & GOBI_WDS) {
-		gprs = ofono_gprs_create(modem, 0, "qmimodem", data->device);
-		gc = ofono_gprs_context_create(modem, 0, "qmimodem",
-							data->device);
-
-		if (gprs && gc)
-			ofono_gprs_add_context(gprs, gc);
-	}
 }
 
 static struct ofono_modem_driver gobi_driver = {
