@@ -58,6 +58,7 @@ static const char *opcm_prefix[] = { "_OPCMENABLE:", NULL };
 struct hso_data {
 	GAtChat *app;
 	GAtChat *control;
+	GAtChat *modem;
 	guint sim_poll_source;
 	guint sim_poll_count;
 	ofono_bool_t have_sim;
@@ -217,6 +218,16 @@ static void sim_status(gboolean ok, GAtResult *result, gpointer user_data)
 	if (data->have_sim == FALSE)
 		return;
 
+	/*
+	 * Ensure that the modem is using GSM character set and not IRA,
+	 * otherwise weirdness with umlauts and other non-ASCII characters
+	 * can result
+	 */
+	g_at_chat_send(data->control, "AT+CSCS=\"GSM\"", none_prefix,
+							NULL, NULL, NULL);
+	g_at_chat_send(data->app, "AT+CSCS=\"GSM\"", none_prefix,
+							NULL, NULL, NULL);
+
 	/*
 	 * Option has the concept of Speech Service versus
 	 * Data Service. Problem is that in Data Service mode
@@ -287,60 +298,30 @@ static void cfun_enable(gboolean ok, GAtResult *result, gpointer user_data)
 					check_model, modem, NULL);
 }
 
-static GAtChat *create_port(const char *device)
+static GAtChat *open_device(struct ofono_modem *modem,
+				const char *key, char *debug)
 {
-	GAtSyntax *syntax;
-	GIOChannel *channel;
-	GAtChat *chat;
-
-	channel = g_at_tty_open(device, NULL);
-	if (channel == NULL)
-		return NULL;
-
-	syntax = g_at_syntax_new_gsm_permissive();
-	chat = g_at_chat_new(channel, syntax);
-	g_at_syntax_unref(syntax);
-	g_io_channel_unref(channel);
-
-	if (chat == NULL)
-		return NULL;
-
-	return chat;
+	return at_util_open_device(modem, key, hso_debug, debug, NULL);
 }
 
 static int hso_enable(struct ofono_modem *modem)
 {
 	struct hso_data *data = ofono_modem_get_data(modem);
-	const char *app;
-	const char *control;
 
 	DBG("%p", modem);
 
-	control = ofono_modem_get_string(modem, "ControlPort");
-	app = ofono_modem_get_string(modem, "ApplicationPort");
-
-	if (app == NULL || control == NULL)
+	data->control = open_device(modem, "Control", "Control: ");
+	if (data->control == NULL)
 		return -EINVAL;
 
-	data->control = create_port(control);
-
-	if (data->control == NULL)
-		return -EIO;
-
-	if (getenv("OFONO_AT_DEBUG"))
-		g_at_chat_set_debug(data->control, hso_debug, "Control: ");
-
-	data->app = create_port(app);
-
+	data->app = open_device(modem, "Application", "App: ");
 	if (data->app == NULL) {
 		g_at_chat_unref(data->control);
 		data->control = NULL;
-
 		return -EIO;
 	}
 
-	if (getenv("OFONO_AT_DEBUG"))
-		g_at_chat_set_debug(data->app, hso_debug, "App: ");
+	data->modem = open_device(modem, "Modem", "Modem: ");
 
 	g_at_chat_send(data->control, "ATE0 +CMEE=1", NULL, NULL, NULL, NULL);
 	g_at_chat_send(data->app, "ATE0 +CMEE=1", NULL, NULL, NULL, NULL);
@@ -377,6 +358,9 @@ static int hso_disable(struct ofono_modem *modem)
 	g_at_chat_cancel_all(data->control);
 	g_at_chat_unregister_all(data->control);
 
+	g_at_chat_unref(data->modem);
+	data->modem = NULL;
+
 	g_at_chat_unref(data->app);
 	data->app = NULL;
 
@@ -410,9 +394,9 @@ static void hso_set_online(struct ofono_modem *modem, ofono_bool_t online,
 	if (g_at_chat_send(chat, command, NULL, set_online_cb, cbd, g_free))
 		return;
 
-	g_free(cbd);
-
 	CALLBACK_WITH_FAILURE(cb, cbd->data);
+
+	g_free(cbd);
 }
 
 static void hso_pre_sim(struct ofono_modem *modem)
