@@ -410,29 +410,32 @@ static void phonesim_query_available_rats(struct ofono_radio_settings *rs,
 }
 
 static struct ofono_gprs_context_driver context_driver = {
-	.name			= "phonesim",
 	.probe			= phonesim_context_probe,
 	.remove			= phonesim_context_remove,
 	.activate_primary	= phonesim_activate_primary,
 	.deactivate_primary	= phonesim_deactivate_primary,
 };
 
+OFONO_ATOM_DRIVER_BUILTIN(gprs_context, phonesim, &context_driver)
+
 static struct ofono_radio_settings_driver radio_settings_driver = {
-	.name			= "phonesim",
 	.probe			= phonesim_radio_settings_probe,
 	.remove			= phonesim_radio_settings_remove,
 	.query_rat_mode		= phonesim_query_rat_mode,
 	.query_available_rats	= phonesim_query_available_rats,
 };
 
+OFONO_ATOM_DRIVER_BUILTIN(radio_settings, phonesim, &radio_settings_driver)
+
 static struct ofono_ctm_driver ctm_driver = {
-	.name			= "phonesim",
 	.probe			= phonesim_ctm_probe,
 	.remove			= phonesim_ctm_remove,
 	.query_tty		= phonesim_ctm_query,
 	.set_tty		= phonesim_ctm_set,
 };
 
+OFONO_ATOM_DRIVER_BUILTIN(ctm, phonesim, &ctm_driver)
+
 static int phonesim_probe(struct ofono_modem *modem)
 {
 	struct phonesim_data *data;
@@ -861,6 +864,7 @@ static void phonesim_post_sim(struct ofono_modem *modem)
 
 	ofono_radio_settings_create(modem, 0, "phonesim", data->chat);
 	ofono_sim_auth_create(modem);
+	ofono_lte_create(modem, 0, "atmodem", data->chat);
 }
 
 static void phonesim_post_online(struct ofono_modem *modem)
@@ -1149,10 +1153,6 @@ static int phonesim_init(void)
 
 	ofono_modem_driver_register(&localhfp_driver);
 
-	ofono_gprs_context_driver_register(&context_driver);
-	ofono_ctm_driver_register(&ctm_driver);
-	ofono_radio_settings_driver_register(&radio_settings_driver);
-
 	if (conf_override)
 		parse_config(conf_override);
 	else
@@ -1174,11 +1174,6 @@ static void phonesim_exit(void)
 	g_slist_free(modem_list);
 	modem_list = NULL;
 
-	ofono_radio_settings_driver_unregister(&radio_settings_driver);
-	ofono_ctm_driver_unregister(&ctm_driver);
-
-	ofono_gprs_context_driver_unregister(&context_driver);
-
 	ofono_modem_driver_unregister(&phonesim_driver);
 }
 
