@@ -80,8 +80,6 @@ struct cf_ss_request {
 	GSList *cf_list[4];
 };
 
-static GSList *g_drivers = NULL;
-
 static void get_query_next_cf_cond(struct ofono_call_forwarding *cf);
 static void set_query_next_cf_cond(struct ofono_call_forwarding *cf);
 static void ss_set_query_next_cf_cond(struct ofono_call_forwarding *cf);
@@ -1437,27 +1435,6 @@ static void sim_read_cf_indicator(struct ofono_call_forwarding *cf)
 	}
 }
 
-int ofono_call_forwarding_driver_register(
-				const struct ofono_call_forwarding_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	if (d->probe == NULL)
-		return -EINVAL;
-
-	g_drivers = g_slist_prepend(g_drivers, (void *) d);
-
-	return 0;
-}
-
-void ofono_call_forwarding_driver_unregister(
-				const struct ofono_call_forwarding_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	g_drivers = g_slist_remove(g_drivers, (void *) d);
-}
-
 static void call_forwarding_remove(struct ofono_atom *atom)
 {
 	struct ofono_call_forwarding *cf = __ofono_atom_get_data(atom);
@@ -1475,40 +1452,7 @@ static void call_forwarding_remove(struct ofono_atom *atom)
 	g_free(cf);
 }
 
-struct ofono_call_forwarding *ofono_call_forwarding_create(
-						struct ofono_modem *modem,
-						unsigned int vendor,
-						const char *driver, void *data)
-{
-	struct ofono_call_forwarding *cf;
-	GSList *l;
-
-	if (driver == NULL)
-		return NULL;
-
-	cf = g_try_new0(struct ofono_call_forwarding, 1);
-
-	if (cf == NULL)
-		return NULL;
-
-	cf->atom = __ofono_modem_add_atom(modem,
-						OFONO_ATOM_TYPE_CALL_FORWARDING,
-						call_forwarding_remove, cf);
-	for (l = g_drivers; l; l = l->next) {
-		const struct ofono_call_forwarding_driver *drv = l->data;
-
-		if (g_strcmp0(drv->name, driver))
-			continue;
-
-		if (drv->probe(cf, vendor, data) < 0)
-			continue;
-
-		cf->driver = drv;
-		break;
-	}
-
-	return cf;
-}
+OFONO_DEFINE_ATOM_CREATE(call_forwarding, OFONO_ATOM_TYPE_CALL_FORWARDING)
 
 static void ussd_watch(struct ofono_atom *atom,
 			enum ofono_atom_watch_condition cond, void *data)
