@@ -28,6 +28,7 @@
 #include <errno.h>
 
 #include <glib.h>
+#include <ell/ell.h>
 
 #include <ofono/types.h>
 #include <ofono/gprs-context.h>
@@ -283,30 +284,6 @@ gboolean valid_long_phone_number_format(const char *number)
 	return valid_number_format(number, OFONO_MAX_PHONE_NUMBER_LENGTH);
 }
 
-gboolean valid_cdma_phone_number_format(const char *number)
-{
-	int len = strlen(number);
-	int i;
-
-	if (!len)
-		return FALSE;
-
-	if (len > OFONO_CDMA_MAX_PHONE_NUMBER_LENGTH)
-		return FALSE;
-
-	for (i = 0; i < len; i++) {
-		if (number[i] >= '0' && number[i] <= '9')
-			continue;
-
-		if (number[i] == '*' || number[i] == '#')
-			continue;
-
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
 const char *telephony_error_to_str(const struct ofono_error *error)
 {
 	struct error_entry *e;
@@ -450,23 +427,6 @@ void string_to_phone_number(const char *str, struct ofono_phone_number *ph)
 	}
 }
 
-const char *cdma_phone_number_to_string(
-				const struct ofono_cdma_phone_number *ph)
-{
-	static char buffer[OFONO_CDMA_MAX_PHONE_NUMBER_LENGTH + 1];
-
-	strncpy(buffer, ph->number, OFONO_CDMA_MAX_PHONE_NUMBER_LENGTH);
-	buffer[OFONO_CDMA_MAX_PHONE_NUMBER_LENGTH] = '\0';
-
-	return buffer;
-}
-
-void string_to_cdma_phone_number(const char *str,
-					struct ofono_cdma_phone_number *ph)
-{
-	strcpy(ph->number, str);
-}
-
 gboolean valid_ussd_string(const char *str, gboolean call_in_progress)
 {
 	int len = strlen(str);
@@ -788,20 +748,20 @@ const char *gprs_proto_to_string(enum ofono_gprs_proto proto)
 	return NULL;
 }
 
-gboolean gprs_proto_from_string(const char *str, enum ofono_gprs_proto *proto)
+bool gprs_proto_from_string(const char *str, enum ofono_gprs_proto *proto)
 {
-	if (g_str_equal(str, "ip")) {
+	if (l_streq0(str, "ip")) {
 		*proto = OFONO_GPRS_PROTO_IP;
-		return TRUE;
-	} else if (g_str_equal(str, "ipv6")) {
+		return true;
+	} else if (l_streq0(str, "ipv6")) {
 		*proto = OFONO_GPRS_PROTO_IPV6;
-		return TRUE;
+		return true;
 	} else if (g_str_equal(str, "dual")) {
 		*proto = OFONO_GPRS_PROTO_IPV4V6;
-		return TRUE;
+		return true;
 	}
 
-	return FALSE;
+	return false;
 }
 
 const char *gprs_auth_method_to_string(enum ofono_gprs_auth_method auth)
@@ -818,19 +778,19 @@ const char *gprs_auth_method_to_string(enum ofono_gprs_auth_method auth)
 	return NULL;
 }
 
-gboolean gprs_auth_method_from_string(const char *str,
+bool gprs_auth_method_from_string(const char *str,
 					enum ofono_gprs_auth_method *auth)
 {
-	if (g_str_equal(str, "chap")) {
+	if (l_streq0(str, "chap")) {
 		*auth = OFONO_GPRS_AUTH_METHOD_CHAP;
-		return TRUE;
-	} else if (g_str_equal(str, "pap")) {
+		return true;
+	} else if (l_streq0(str, "pap")) {
 		*auth = OFONO_GPRS_AUTH_METHOD_PAP;
-		return TRUE;
-	} else if (g_str_equal(str, "none")) {
+		return true;
+	} else if (l_streq0(str, "none")) {
 		*auth = OFONO_GPRS_AUTH_METHOD_NONE;
-		return TRUE;
+		return true;
 	}
 
-	return FALSE;
+	return false;
 }
