@@ -31,12 +31,11 @@
 #include <gdbus.h>
 
 #include "ofono.h"
-
 #include "common.h"
+#include "missing.h"
 
 #define DEFAULT_POWERED_TIMEOUT (20)
 
-static GSList *g_devinfo_drivers;
 static GSList *g_driver_list;
 static GSList *g_modem_list;
 
@@ -457,6 +456,28 @@ static void flush_atoms(struct ofono_modem *modem, enum modem_state new_state)
 	}
 }
 
+const void *__ofono_atom_driver_builtin_find(const char *name,
+				const struct ofono_atom_driver_desc *start,
+				const struct ofono_atom_driver_desc *stop)
+{
+	const struct ofono_atom_driver_desc *desc;
+
+	if (!name)
+		return NULL;
+
+	for (desc = start; desc < stop; desc++) {
+		if (!desc->name || !desc->driver)
+			continue;
+
+		if (strcmp(desc->name, name))
+			continue;
+
+		return desc->driver;
+	}
+
+	return NULL;
+}
+
 static void notify_online_watches(struct ofono_modem *modem)
 {
 	struct ofono_watchlist_item *item;
@@ -1603,25 +1624,6 @@ static void dun_watch(struct ofono_atom *atom,
 	ofono_emulator_add_handler(em, "+GCAP", gcap_cb, data, NULL);
 }
 
-int ofono_devinfo_driver_register(const struct ofono_devinfo_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	if (d->probe == NULL)
-		return -EINVAL;
-
-	g_devinfo_drivers = g_slist_prepend(g_devinfo_drivers, (void *) d);
-
-	return 0;
-}
-
-void ofono_devinfo_driver_unregister(const struct ofono_devinfo_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	g_devinfo_drivers = g_slist_remove(g_devinfo_drivers, (void *) d);
-}
-
 static void devinfo_remove(struct ofono_atom *atom)
 {
 	struct ofono_devinfo *info = __ofono_atom_get_data(atom);
@@ -1639,34 +1641,7 @@ static void devinfo_remove(struct ofono_atom *atom)
 	g_free(info);
 }
 
-struct ofono_devinfo *ofono_devinfo_create(struct ofono_modem *modem,
-							unsigned int vendor,
-							const char *driver,
-							void *data)
-{
-	struct ofono_devinfo *info;
-	GSList *l;
-
-	info = g_new0(struct ofono_devinfo, 1);
-
-	info->atom = __ofono_modem_add_atom(modem, OFONO_ATOM_TYPE_DEVINFO,
-						devinfo_remove, info);
-
-	for (l = g_devinfo_drivers; l; l = l->next) {
-		const struct ofono_devinfo_driver *drv = l->data;
-
-		if (g_strcmp0(drv->name, driver))
-			continue;
-
-		if (drv->probe(info, vendor, data) < 0)
-			continue;
-
-		info->driver = drv;
-		break;
-	}
-
-	return info;
-}
+OFONO_DEFINE_ATOM_CREATE(devinfo, OFONO_ATOM_TYPE_DEVINFO)
 
 static void devinfo_unregister(struct ofono_atom *atom)
 {
@@ -1754,10 +1729,10 @@ static int set_modem_property(struct ofono_modem *modem, const char *name,
 		property->value = g_strdup((const char *) value);
 		break;
 	case PROPERTY_TYPE_INTEGER:
-		property->value = g_memdup(value, sizeof(int));
+		property->value = g_memdup2(value, sizeof(int));
 		break;
 	case PROPERTY_TYPE_BOOLEAN:
-		property->value = g_memdup(value, sizeof(ofono_bool_t));
+		property->value = g_memdup2(value, sizeof(ofono_bool_t));
 		break;
 	default:
 		break;
@@ -1943,12 +1918,13 @@ static void sim_watch(struct ofono_atom *atom,
 							modem, NULL);
 }
 
-void __ofono_modemwatch_init(void)
+static int modemwatch_init(void)
 {
 	g_modemwatches = __ofono_watchlist_new(g_free);
+	return 0;
 }
 
-void __ofono_modemwatch_cleanup(void)
+static void modemwatch_cleanup(void)
 {
 	__ofono_watchlist_free(g_modemwatches);
 }
@@ -2344,3 +2320,5 @@ void __ofono_modem_dec_emergency_mode(struct ofono_modem *modem)
 out:
 	modem->emergency--;
 }
+
+OFONO_MODULE(modemwatch, modemwatch_init, modemwatch_cleanup)
