@@ -77,8 +77,7 @@ void __ofono_nettime_probe_drivers(struct ofono_modem *modem)
 		driver = l->data;
 
 		context = nettime_context_create(modem, driver);
-
-		if (!context)
+		if (context == NULL)
 			continue;
 
 		__ofono_modem_add_atom(modem, OFONO_ATOM_TYPE_NETTIME,
