@@ -44,8 +44,6 @@
 #include "stkagent.h"
 #include "util.h"
 
-static GSList *g_drivers = NULL;
-
 struct stk_timer {
 	time_t expiry;
 	time_t start;
@@ -3114,25 +3112,6 @@ out:
 	}
 }
 
-int ofono_stk_driver_register(const struct ofono_stk_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	if (d->probe == NULL)
-		return -EINVAL;
-
-	g_drivers = g_slist_prepend(g_drivers, (void *) d);
-
-	return 0;
-}
-
-void ofono_stk_driver_unregister(const struct ofono_stk_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	g_drivers = g_slist_remove(g_drivers, (void *) d);
-}
-
 static void stk_unregister(struct ofono_atom *atom)
 {
 	struct ofono_stk *stk = __ofono_atom_get_data(atom);
@@ -3186,40 +3165,7 @@ static void stk_remove(struct ofono_atom *atom)
 	g_free(stk);
 }
 
-struct ofono_stk *ofono_stk_create(struct ofono_modem *modem,
-					unsigned int vendor,
-					const char *driver,
-					void *data)
-{
-	struct ofono_stk *stk;
-	GSList *l;
-
-	if (driver == NULL)
-		return NULL;
-
-	stk = g_try_new0(struct ofono_stk, 1);
-
-	if (stk == NULL)
-		return NULL;
-
-	stk->atom = __ofono_modem_add_atom(modem, OFONO_ATOM_TYPE_STK,
-						stk_remove, stk);
-
-	for (l = g_drivers; l; l = l->next) {
-		const struct ofono_stk_driver *drv = l->data;
-
-		if (g_strcmp0(drv->name, driver))
-			continue;
-
-		if (drv->probe(stk, vendor, data) < 0)
-			continue;
-
-		stk->driver = drv;
-		break;
-	}
-
-	return stk;
-}
+OFONO_DEFINE_ATOM_CREATE(stk, OFONO_ATOM_TYPE_STK)
 
 void ofono_stk_register(struct ofono_stk *stk)
 {
