@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -42,6 +42,7 @@
 #define OFONO_MODEM_INTERFACE		OFONO_SERVICE ".Modem"
 #define OFONO_CALLMANAGER_INTERFACE	OFONO_SERVICE ".VoiceCallManager"
 #define OFONO_CALL_INTERFACE		OFONO_SERVICE ".VoiceCall"
+#define OFONO_AUDIO_INTERFACE		OFONO_SERVICE ".AudioSettings"
 
 struct modem_data {
 	char *path;
@@ -51,8 +52,10 @@ struct modem_data {
 	guint call_added_watch;
 	guint call_removed_watch;
 	guint call_changed_watch;
+	guint audio_changed_watch;
 
 	gboolean has_callmanager;
+	gboolean has_audiosettings;
 	gboolean is_huawei;
 	gint audio_users;
 	guint audio_watch;
@@ -90,6 +93,10 @@ static gboolean audio_receive(GIOChannel *channel,
 		return TRUE;
 
 	wlen = write(modem->dsp_out, buf, rlen);
+	if (wlen < 0) {
+		modem->audio_watch = 0;
+		return FALSE;
+	}
 
 	return TRUE;
 }
@@ -178,6 +185,25 @@ static void close_audio(struct modem_data *modem)
 	close(modem->dsp_out);
 }
 
+static void audio_set(struct modem_data *modem, const char *key,
+						DBusMessageIter *iter)
+{
+	const char *str = NULL;
+
+	if (dbus_message_iter_get_arg_type(iter) == DBUS_TYPE_STRING)
+		dbus_message_iter_get_basic(iter, &str);
+
+	if (dbus_message_iter_get_arg_type(iter) == DBUS_TYPE_BOOLEAN) {
+		dbus_bool_t val;
+
+		dbus_message_iter_get_basic(iter, &val);
+		str = (val == TRUE) ? "yes" : "no";
+	}
+
+	g_print("updating audio (%s) [ %s = %s ]\n", modem->path,
+						key, str ? str : "...");
+}
+
 static void call_set(struct call_data *call, const char *key,
 						DBusMessageIter *iter)
 {
@@ -311,6 +337,26 @@ static gboolean call_changed(DBusConnection *conn,
 	return TRUE;
 }
 
+static gboolean audio_changed(DBusConnection *conn,
+				DBusMessage *msg, void *user_data)
+{
+	struct modem_data *modem = user_data;
+	DBusMessageIter iter, value;
+	const char *key;
+
+	if (dbus_message_iter_init(msg, &iter) == FALSE)
+		return TRUE;
+
+	dbus_message_iter_get_basic(&iter, &key);
+
+	dbus_message_iter_next(&iter);
+	dbus_message_iter_recurse(&iter, &value);
+
+	audio_set(modem, key, &value);
+
+	return TRUE;
+}
+
 static void get_calls_reply(DBusPendingCall *call, void *user_data)
 {
 	struct modem_data *modem = user_data;
@@ -389,6 +435,7 @@ static void check_interfaces(struct modem_data *modem, DBusMessageIter *iter)
 {
 	DBusMessageIter entry;
 	gboolean has_callmanager = FALSE;
+	gboolean has_audiosettings = FALSE;
 
 	dbus_message_iter_recurse(iter, &entry);
 
@@ -400,9 +447,14 @@ static void check_interfaces(struct modem_data *modem, DBusMessageIter *iter)
 		if (g_str_equal(interface, OFONO_CALLMANAGER_INTERFACE) == TRUE)
 			has_callmanager = TRUE;
 
+		if (g_str_equal(interface, OFONO_AUDIO_INTERFACE) == TRUE)
+			has_audiosettings = TRUE;
+
 		dbus_message_iter_next(&entry);
 	}
 
+	modem->has_audiosettings = has_audiosettings;
+
 	if (modem->has_callmanager == has_callmanager)
 		return;
 
@@ -430,6 +482,7 @@ static void destroy_modem(gpointer data)
 	g_dbus_remove_watch(modem->conn, modem->call_added_watch);
 	g_dbus_remove_watch(modem->conn, modem->call_removed_watch);
 	g_dbus_remove_watch(modem->conn, modem->call_changed_watch);
+	g_dbus_remove_watch(modem->conn, modem->audio_changed_watch);
 
 	g_hash_table_destroy(modem->call_list);
 
@@ -461,15 +514,19 @@ static void create_modem(DBusConnection *conn,
 
 	modem->conn = conn;
 
-	modem->call_added_watch = g_dbus_add_signal_watch(conn, NULL,
+	modem->call_added_watch = g_dbus_add_signal_watch(conn, OFONO_SERVICE,
 				modem->path, OFONO_CALLMANAGER_INTERFACE,
 				"CallAdded", call_added, modem, NULL);
-	modem->call_removed_watch = g_dbus_add_signal_watch(conn, NULL,
-				modem->path, OFONO_CALLMANAGER_INTERFACE,
-				"CallRemoved", call_removed, modem, NULL);
-	modem->call_changed_watch = g_dbus_add_signal_watch(conn, NULL,
-				NULL, OFONO_CALL_INTERFACE,
+	modem->call_removed_watch = g_dbus_add_signal_watch(conn,
+				OFONO_SERVICE, modem->path,
+				OFONO_CALLMANAGER_INTERFACE, "CallRemoved",
+				call_removed, modem, NULL);
+	modem->call_changed_watch = g_dbus_add_signal_watch(conn,
+				OFONO_SERVICE, NULL, OFONO_CALL_INTERFACE,
 				"PropertyChanged", call_changed, modem, NULL);
+	modem->audio_changed_watch = g_dbus_add_signal_watch(conn,
+				OFONO_SERVICE, NULL, OFONO_AUDIO_INTERFACE,
+				"PropertyChanged", audio_changed, modem, NULL);
 
 	g_hash_table_replace(modem_list, modem->path, modem);
 
@@ -718,11 +775,6 @@ int main(int argc, char **argv)
 	guint watch;
 	struct sigaction sa;
 
-#ifdef NEED_THREADS
-	if (g_thread_supported() == FALSE)
-		g_thread_init(NULL);
-#endif
-
 	context = g_option_context_new(NULL);
 	g_option_context_add_main_entries(context, options, NULL);
 
@@ -744,13 +796,6 @@ int main(int argc, char **argv)
 
 	main_loop = g_main_loop_new(NULL, FALSE);
 
-#ifdef NEED_THREADS
-	if (dbus_threads_init_default() == FALSE) {
-		fprintf(stderr, "Can't init usage of threads\n");
-		exit(1);
-	}
-#endif
-
 	dbus_error_init(&err);
 
 	conn = g_dbus_setup_bus(DBUS_BUS_SYSTEM, NULL, &err);
